from pymongo.errors import DuplicateKeyError
from marshmallow.exceptions import ValidationError
from umongo import Document, fields
from umongo.instance import Instance
import pytest
from mongomock_motor import AsyncMongoMockClient


@pytest.mark.anyio
async def test_umongo():
    instance = Instance.from_db(AsyncMongoMockClient()['tests'])

    @instance.register
    class Something(Document):
        field1 = fields.StrField(required=True, unique=True)
        field2 = fields.StrField()
        related = fields.ListField(fields.ReferenceField("Something"))

        class Meta:
            collection_name = "something"

    await Something.ensure_indexes()

    something1 = Something(field1='A', field2=':)', related=[])
    await something1.commit()

    with pytest.raises(ValidationError, match="^{'field1': 'Field value must be unique.'}$"):
        duplicate = Something(field1='A', field2=':)', related=[])
        await duplicate.commit()

    something2 = Something(field1='B', field2=';)', related=[something1])
    await something2.commit()

    found = await Something.find_one({'field1': 'B'})
    assert found
    assert found.field1 == 'B'
    assert found.field2 == ';)'
    assert len(found.related) == 1

    first_related = await found.related[0].fetch()
    assert first_related.field1 == 'A'
    assert first_related.field2 == ':)'
    assert first_related.related == []



import bson
import pytest
from datetime import datetime, timezone
from mongomock_motor import AsyncMongoMockClient


@pytest.mark.anyio
async def test_workflow():
    collection = AsyncMongoMockClient()['tests']['test']

    result = await collection.insert_one({'a': 1})
    assert result.inserted_id
    doc_id = result.inserted_id

    result = await collection.update_one({'a': 1}, {'$set': {'b': 1}, '$inc': {'a': 1}})
    assert result.modified_count == 1

    result = await collection.update_one({'a': 1}, {'$set': {'b': 100}})
    assert result.modified_count == 0

    result = await collection.update_one({'_id': doc_id}, {'$inc': {'a': 1}})
    assert result.modified_count == 1

    docs = await collection.find({'a': 3}).to_list(None)
    assert len(docs) == 1
    assert docs[0]['_id'] == doc_id
    assert docs[0]['a'] == 3
    assert docs[0]['b'] == 1

    pipeline = [{"$match": {"a": 3}}]
    docs = await collection.aggregate(pipeline).to_list(None)
    assert len(docs) == 1
    assert docs[0]['_id'] == doc_id
    assert docs[0]['a'] == 3
    assert docs[0]['b'] == 1


@pytest.mark.anyio
async def test_tz_awareness():
    tz_aware_date = datetime(2022, 4, 26, tzinfo=timezone.utc)
    
    # Naive
    collection = AsyncMongoMockClient()['tests']['test']
    await collection.insert_one({'d': tz_aware_date})
    result = await collection.find_one()
    assert result["d"].tzinfo is None

    # Aware
    collection = AsyncMongoMockClient(tz_aware=True)['tests']['test']
    await collection.insert_one({'d': tz_aware_date})
    result = await collection.find_one()
    assert result["d"].tzinfo.__class__ is bson.tz_util.FixedOffset




import pymongo
import pytest
from mongomock_motor import AsyncMongoMockClient


EXPECTED_DOCUMENTS_COUNT = 10


@pytest.mark.anyio
async def test_skip_and_limit():
    collection = AsyncMongoMockClient()['tests']['test']

    # Insert sample documents into database
    await collection.insert_many([{'i': i} for i in range(EXPECTED_DOCUMENTS_COUNT)])

    # Query without limitations
    docs = await collection.find().to_list()
    assert len(docs) == EXPECTED_DOCUMENTS_COUNT

    # Query with limit
    docs = await collection.find().limit(2).to_list()
    assert len(docs) == 2

    # Query with skip
    docs = await collection.find().skip(2).to_list()
    assert len(docs) == EXPECTED_DOCUMENTS_COUNT - 2

    # Query with limit and skip
    docs = await collection.find().skip(2).limit(2).to_list()
    assert len(docs) == 2

    # Query with limit, skip and sort
    docs = await (
        collection
        .find(projection={'_id': 0})
        .skip(2)
        .limit(2)
        .sort('i', pymongo.DESCENDING)
        .to_list()
    )

    assert len(docs) == 2
    assert docs == [
        {'i': EXPECTED_DOCUMENTS_COUNT - 3},
        {'i': EXPECTED_DOCUMENTS_COUNT - 4},
    ]



from mongomock import DuplicateKeyError, helpers


def _provide_error_details(collection, data, exception):
    if not isinstance(exception, DuplicateKeyError):
        return exception

    for index in collection._store.indexes.values():
        if not index.get('unique'):
            continue

        is_sparse = index.get('sparse')

        find_kwargs = {}
        for key, _ in index.get('key'):
            try:
                find_kwargs[key] = helpers.get_value_by_dot(data, key)
            except KeyError:
                find_kwargs[key] = None

        if is_sparse and set(find_kwargs.values()) == {None}:
            continue

        found_documents = list(collection._iter_documents(find_kwargs))
        if len(found_documents) > 0:
            return DuplicateKeyError(
                'E11000 Duplicate Key Error',
                11000,
                {
                    'keyValue': find_kwargs,
                    'keyPattern': dict(index.get('key')),
                },
                None,
            )

    return exception


def _patch_collection_internals(collection):
    """
    # Details for DuplicateKeyError

    Adds details with 'keyPattern' and 'keyValue' when
    raising DuplicateKeyError from _insert
    https://github.com/mongomock/mongomock/issues/773
    """
    _insert = collection._insert

    def insert(data, *args, **kwargs):
        try:
            return _insert(data, *args, **kwargs)
        except DuplicateKeyError as exc:
            raise _provide_error_details(collection, data, exc)

    collection._insert = insert

    return collection



import pytest
from pymongo.errors import DuplicateKeyError
from mongomock_motor import AsyncMongoMockClient


@pytest.mark.anyio
async def test_unique_while_inserting():
    collection = AsyncMongoMockClient()['tests']['test']

    await collection.create_index('a', unique=True)

    await collection.insert_one({'a': 1})

    with pytest.raises(DuplicateKeyError):
        await collection.insert_one({'a': 1})

    assert len(await collection.find({}).to_list(None)) == 1


@pytest.mark.anyio
async def test_unique_while_creating_index():
    collection = AsyncMongoMockClient()['tests']['test']

    await collection.insert_one({'a': 1})
    await collection.insert_one({'a': 1})

    with pytest.raises(DuplicateKeyError):
        await collection.create_index('a', unique=True)

    assert len(await collection.find({}).to_list(None)) == 2



import pytest
from mongomock_motor import AsyncMongoMockClient


@pytest.mark.anyio
async def test_mock_client():
    collection = AsyncMongoMockClient()['tests']['test']

    assert await collection.find({}).to_list(None) == []

    result = await collection.insert_one({'a': 1})
    assert result.inserted_id

    assert len(await collection.find({}).to_list(None)) == 1



# -*- coding: utf-8 -*-

"""
:copyright: (c) 2021 by Michael Krukov
:license: MIT, see LICENSE for more details.
"""

import setuptools


VERSION = "0.0.8"


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="mongomock_motor",
    version=VERSION,
    author="Michael Krukov",
    author_email="krukov.michael@ya.ru",
    keywords=["library", "mongodb"],
    description="Library for mocking AsyncIOMotorClient built on top of mongomock.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/michaelkryukov/mongomock_motor",
    packages=setuptools.find_packages(),
    install_requires=[
        "mongomock>=3.23.0,<5.0.0",
    ],
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
)
